/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: api.h,v 1.3 2001/01/31 09:11:29 adam Exp $
 */
#ifndef __API_H__
#define __API_H__

#include "opt.h"
#include "pbuf.h"
#include "sys.h"

#include "ip.h"

#include "udp.h"
#include "tcp.h"

#define NETCONN_NOCOPY 0x00
#define NETCONN_COPY   0x01

enum netconn_type {
  NETCONN_TCP,
  NETCONN_UDP,
  NETCONN_UDPLITE,
  NETCONN_UDPNOCHKSUM
};

struct netbuf {
  struct pbuf *p, *ptr;
  struct ip_addr *fromaddr;
  unsigned short fromport;
};

struct netconn {
  enum netconn_type type;
  union {
    struct tcp_pcb *tcp;
    struct udp_pcb *udp;
  } pcb;
  sys_mbox_t recvmbox;
  sys_mbox_t acceptmbox;
};

/* Network buffer functions: */
struct netbuf *   netbuf_new      (void);
void              netbuf_delete   (struct netbuf *buf);
void *            netbuf_alloc    (struct netbuf *buf, int size);
int               netbuf_free     (struct netbuf *buf);
int               netbuf_ref      (struct netbuf *buf, void *data, int size);
void              netbuf_chain    (struct netbuf *head, struct netbuf *tail);

int               netbuf_len      (struct netbuf *buf);
int               netbuf_data     (struct netbuf *buf, void **data, int *len);
int               netbuf_next     (struct netbuf *buf);
void              netbuf_first    (struct netbuf *buf);

void              netbuf_copy     (struct netbuf *buf, void *data, int len);
struct ip_addr *  netbuf_fromaddr (struct netbuf *buf);
unsigned short    netbuf_fromport (struct netbuf *buf);

/* Network connection functions: */
struct netconn *  netconn_new     (enum netconn_type type);
void              netconn_delete  (struct netconn *conn);
enum netconn_type netconn_type    (struct netconn *conn);
int               netconn_peer    (struct netconn *conn,
				   struct ip_addr **addr,
				   unsigned short *port);
int               netconn_addr    (struct netconn *conn,
				   struct ip_addr **addr,
				   unsigned short *port);
int               netconn_bind    (struct netconn *conn, struct ip_addr *addr,
				   unsigned short port);
int               netconn_connect (struct netconn *conn, struct ip_addr *addr,
				   unsigned short port);
int               netconn_listen  (struct netconn *conn);
struct netconn *  netconn_accept  (struct netconn *conn);
struct netbuf *   netconn_recv    (struct netconn *conn);
int               netconn_send    (struct netconn *conn, struct netbuf *buf);
int               netconn_write   (struct netconn *conn, void *data, int size,
				   unsigned char copy);
int               netconn_close   (struct netconn *conn);


#endif /* __API_H__ */


