/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: ip_addr.h,v 1.5 2001/01/31 09:12:01 adam Exp $
 */
#ifndef __IP_ADDR_H__
#define __IP_ADDR_H__

#include "arch.h"

#define IP_ADDR_ANY 0

struct ip_addr {
  u32_t addr;
};

#define IP4_ADDR(ipaddr, a,b,c,d) (ipaddr)->addr = htonl(((a & 0xff) << 24) | ((b & 0xff) << 16) | \
                                                         ((c & 0xff) << 8) | (d & 0xff))
#define IP6_ADDR \
                   #error "IP6_ADDR not avaliable when compiling with IPv4 support"

#define ip_addr_set(dest, src) (dest)->addr = (src)->addr
#define ip_addr_maskcmp(addr1, addr2, mask) (((addr1)->addr & \
                                              (mask)->addr) == \
                                             ((addr2)->addr & \
                                              (mask)->addr))
#define ip_addr_cmp(addr1, addr2) ((addr1)->addr == (addr2)->addr)

#define ip_addr_isany(addr1) ((addr1) == NULL || (addr1)->addr == 0)

#define ip_addr_debug_print(ipaddr) DEBUGF(DEBUG, ("%d.%d.%d.%d", \
		    (unsigned char)(ntohl((ipaddr)->addr) >> 24) & 0xff, \
		    (unsigned char)(ntohl((ipaddr)->addr) >> 16) & 0xff, \
		    (unsigned char)(ntohl((ipaddr)->addr) >> 8) & 0xff, \
		    (unsigned char)ntohl((ipaddr)->addr) & 0xff))

#endif /* __IP_ADDR_H__ */

