/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: http_noapi.c,v 1.5 2001/01/31 09:12:07 adam Exp $
 */

#include "debug.h"

#include "http_noapi.h"

#include "tcp.h"

#include "fs.h"

/*-----------------------------------------------------------------------------------*/
static void
http_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p)
{
  int i, j;
  char *data;
  char fname[40];
  struct fs_file file;

  
  if(p != NULL) {
    data = p->payload;
    
    if(strncmp(data, "GET ", 4) == 0) {
      for(i = 0; i < 40; i++) {
        if(((char *)data + 4)[i] == ' ' ||
           ((char *)data + 4)[i] == '\r' ||
           ((char *)data + 4)[i] == '\n') {
          ((char *)data + 4)[i] = 0;
        }
      }
      i = 0;
      do {
        fname[i] = "/http"[i];
        i++;
      } while(fname[i - 1] != 0 && i < 40);
      i--;
      j = 0;
      do {
        fname[i] = ((char *)data + 4)[j];
        j++;
        i++;
      } while(fname[i - 1] != 0 && i < 40);
      if(fs_open(fname, &file)) {
        tcp_write(pcb, file.data, file.len, 0);
      }
    }
    pbuf_free(p);      
  }
  tcp_close(pcb);
}
/*-----------------------------------------------------------------------------------*/
static void
http_accept(void *arg, struct tcp_pcb *pcb)
{
  tcp_recv(pcb,
	   (void (*)(void *, struct tcp_pcb *, struct pbuf *))http_recv,
	   NULL);
}
/*-----------------------------------------------------------------------------------*/
void
http_init(void)
{
  struct tcp_pcb *pcb;

  /*  printf("http_init\n");*/
  pcb = tcp_pcb_new();
  /*  printf("http_init: pcb 0x%x\n", (int)pcb);*/
  tcp_bind(pcb, NULL, 90);
  tcp_listen(pcb);
  tcp_accept(pcb, http_accept, NULL);
}
/*-----------------------------------------------------------------------------------*/

