/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: client6.c,v 1.4 2001/01/31 09:12:10 adam Exp $
 */

#include "debug.h"

#include <unistd.h>  /* for pause() */


#include "mem.h"
#include "sys.h"

#include "stats.h"


#include "tcpip.h"

#ifdef linux
#include "tapif.h"
#else /* linux */
#include "tunif.h"
#endif /* linux */

#include "unixif.h"
#include "dropif.h"

#include "loopif.h"

#include "tcpdump.h"

#include "example.h"
#include "udp_example.h"
#include "http_example.h"

#include "http_noapi.h"

#include "systat.h"


#include "perf.h"
/*-----------------------------------------------------------------------------------*/
static void
mem_timeout(void *data)
{
  /*#ifdef MEM_TRACKING
  stats_mem();
#else
  mem_debug_print();
  #endif*/ /* MEM_TRACKING */
#if MEM_DEBUG
  mem_debug_print();
#endif /* MEM_DEBUG */
  sys_timeout(1000, mem_timeout, NULL);
}
/*-----------------------------------------------------------------------------------*/
static void
mem_more_timeout(void *data)
{
#if MEM_DEBUG
#ifdef MEM_TRACKING
  mem_debug_print_more();
#endif /* MEM_TRACKING */
#endif /* MEM_DEBUG */
  sys_timeout(10000, mem_more_timeout, NULL);
}
/*-----------------------------------------------------------------------------------*/
static void
pbuf_timeout(void *data)
{
#if PBUF_DEBUG
  pbuf_debug_print_stats();
#endif /* PBUF_DEBUG */
  /*  mem_debug_print();*/
  sys_timeout(2500, pbuf_timeout, NULL);
}

/*-----------------------------------------------------------------------------------*/
static void
tcp_timeout(void *data)
{
#if TCP_DEBUG
  tcp_debug_print_pcbs();
#endif /* TCP_DEBUG */
  sys_timeout(5000, tcp_timeout, NULL);
}
/*-----------------------------------------------------------------------------------*/
static void
main_thread(void *arg)
{
  struct ip_addr ipaddr, netmask, gw;

  /*gw.addr = htonl(0x7f000001);
  ipaddr.addr = htonl(0x7f000001);
  netmask.addr = htonl(0xff000000);*/

  netif_add(&ipaddr, &netmask, &gw, loopif_init, tcpip_input);
    
  /*  gw.addr = htonl(0x0a000101);
  ipaddr.addr = htonl(0x0a000102);
  netmask.addr = htonl(0xffffff00);*/

  /* For testing without proxy. */

  /*
#ifdef linux
  netif_set_default(netif_add(&ipaddr, &netmask, &gw, tapif_init_thread,
  tcpip_input));
#else
  netif_set_default(netif_add(&ipaddr, &netmask, &gw, tunif_init_thread,
  tcpip_input));
#endif
  */

  /* For testing with proxy. */
#ifdef USE_DROP_INTERFACE
  netif_set_default(netif_add(&ipaddr, &netmask, &gw, dropif_init,
			      tcpip_input));
#else /* USE_DROP_INTERFACE */
  netif_set_default(netif_add(&ipaddr, &netmask, &gw, unixif_init,
			      tcpip_input));
#endif /* USE_DROP_INTERFACE */

 
  
  tcpip_init();
  printf("TCP/IP initialized.\n");

  sys_thread_new(example_thread, NULL);
  sys_thread_new(udp_example_thread, NULL);
  sys_thread_new(http_example_thread, NULL);

  http_init();
  systat_init();

  printf("Applications started.\n");
    
  sys_timeout(1000, mem_timeout, NULL);
  sys_timeout(10000, mem_more_timeout, NULL);
  sys_timeout(5000, tcp_timeout, NULL);
  sys_timeout(1000, pbuf_timeout, NULL);

  sys_main();
}
/*-----------------------------------------------------------------------------------*/
int
main(int argc, char **argv)
{
#ifdef PERF
  perf_init("/tmp/client.perf");
#endif /* PERF */
#ifdef STATS
  stats_init();
#endif /* STATS */
  sys_init();
  mem_init();
  pbuf_init(16, 128);

  tcpdump_init();
  
  printf("System initialized.\n");
    
  sys_thread_new((void *)(main_thread), NULL);
  pause();
  return 0;
}
/*-----------------------------------------------------------------------------------*/
