/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: dropif.c,v 1.5 2001/01/31 09:12:17 adam Exp $
 */
#include "debug.h"

#include <stdlib.h>

#include "opt.h"
#include "def.h"
#include "mem.h"
#include "pbuf.h"
#include "sys.h"

#include "dropif.h"
#include "unixif.h"

#define DROPIF_INPUT_DROPRATE 0.1
#define DROPIF_OUTPUT_DROPRATE 0.1

struct dropif {
  void (* input)(struct pbuf *p, struct netif *inp);
  struct netif *netif;
};

/*-----------------------------------------------------------------------------------*/
static int
dropif_output(struct netif *netif, struct pbuf *p,
	     struct ip_addr *ipaddr)
{
  struct dropif *drp;
  drp = netif->state;

#ifdef DROPIF_OUTPUT_DROPRATE
  if(((double)rand()/(double)RAND_MAX) < DROPIF_OUTPUT_DROPRATE) {
    DEBUGF(DROPIF_DEBUG, ("dropif_output: Packet dropped\n"));
    return 0;
  }
#endif /* DROPIF_OUTPUT_DROPRATE */

  return drp->netif->output(drp->netif, p, ipaddr);
}
/*-----------------------------------------------------------------------------------*/
static void
dropif_input(struct pbuf *p, struct netif *inp)
{
#ifdef DROPIF_INPUT_DROPRATE
  if(((double)rand()/(double)RAND_MAX) < DROPIF_INPUT_DROPRATE) {
    DEBUGF(DROPIF_DEBUG, ("dropif_irq_handler: Packet dropped\n"));
    pbuf_free(p);
    return;
  }
#endif /* DROPIF_INPUT_DROPRATE */

  /* XXX */
  tcpip_input(p, inp);
}
/*-----------------------------------------------------------------------------------*/
void
dropif_init(struct netif *netif)
{
  struct dropif *drp;
  
  drp = malloc(sizeof(struct dropif));  
  netif->state = drp;
  netif->name[0] = 'd';
  netif->name[1] = 'r';
  netif->output = dropif_output;
  
  netif->input = dropif_input;
  
  drp->netif = malloc(sizeof(struct netif));
  unixif_init(drp->netif);
}
/*-----------------------------------------------------------------------------------*/







